/* eslint-disable @typescript-eslint/no-explicit-any */
import {
    Box,
    Grid,
    Input,
    InputAdornment,
    Table,
    TableBody,
    TableCell,
    TableContainer,
    TableHead,
    TableRow,
    Typography,
} from "@mui/material";
// Componentes de Ação
import Loading from "../Loading";
import { DeleteButton as Delete } from "../Buttons/Delete";
import { RedirectButton as Redirect } from "../Buttons/Redirect";
import Paginator, { Pagination } from "./Paginator";
// Placeholders
import SearchIcon from "@mui/icons-material/Search";
import { EmptySearch, Error } from "../Placeholders";

export interface Data {
    [key: string]: string | number | null;
}

interface Cell {
    title: string;
    jsonField: string;
    type?: string;
    width?: string;
    mappedValue?: Function;
}

interface TableProps {
    cells: Cell[];
    tableName: string;
    rows: any[];
    pagination: Pagination;
    loading: boolean;
    error: boolean;
    onPageChange: (pagination: Pagination) => void;
    onDelete?: (id: any) => void;
    onRedirect?: (path: any) => void;
    onSearch: (search: string) => void;
}

const CustomLoading = () => {
    return (
        <div style={{ height: "50vh" }}>
            <Loading />
        </div>
    );
};

export default function CustomTable({
    cells,
    tableName,
    rows,
    pagination,
    loading,
    error,
    onDelete = () => {},
    onRedirect = () => {},
    onPageChange,
    onSearch,
}: Readonly<TableProps>) {
    const dynamicWidth = 100 / cells.length + "%";
    return (
        <Box sx={{ height: "100%" }}>
            <Box>
                <Grid container style={{ padding: 10 }}>
                    <Grid item xs={3}>
                        <Typography variant="h4" fontWeight={"bold"} align="left">
                            {tableName}
                        </Typography>
                    </Grid>
                    <Grid item xs={9}>
                        <Input
                            onChange={(searchVal) => onSearch(searchVal.target.value)}
                            fullWidth
                            placeholder="Procurar por informações na tabela"
                            endAdornment={
                                <InputAdornment position="start">
                                    <SearchIcon />
                                </InputAdornment>
                            }
                        />
                    </Grid>
                </Grid>
            </Box>
            <Box sx={{ display: "flex", flexDirection: "column", height: "100%" }}>
                <TableContainer>
                    {loading ? (
                        <CustomLoading />
                    ) : error ? (
                        <Error />
                    ) : !rows?.length ? (
                        <EmptySearch />
                    ) : (
                        <Table sx={{ maxHeight: "100%" }} aria-label="small table" size="small">
                            <TableHead>
                                <TableRow sx={{ height: "8vh" }}>
                                    {cells.map((cell, cellIndex) => {
                                        return (
                                            <TableCell
                                                key={`table-cell-${cellIndex}`}
                                                sx={{
                                                    fontWeight: "bold",
                                                    width: cell.width ?? dynamicWidth,
                                                }}
                                            >
                                                {cell.title}
                                            </TableCell>
                                        );
                                    })}
                                </TableRow>
                            </TableHead>
                            <TableBody>
                                {rows!.map((row: Data, rowIndex: any) => {
                                    return (
                                        <TableRow
                                            style={{ height: "7vh" }}
                                            key={`table-row-${rowIndex}`}
                                        >
                                            {cells.map((cell, cellIndex) => {
                                                return (
                                                    <TableCell
                                                        sx={{
                                                            width: cell.width ?? dynamicWidth,
                                                            // padding: 15,
                                                        }}
                                                        key={`table-cell-${rowIndex}-${cellIndex}`}
                                                        align="left"
                                                    >
                                                        {cell.type === "delete" && row.id ? (
                                                            <Delete
                                                                handleDelete={() =>
                                                                    onDelete(row.id)
                                                                }
                                                            />
                                                        ) : cell.type === "redirect" && row.number ? (
                                                            <Redirect
                                                                handleRedirect={() =>
                                                                    onRedirect(row.number)
                                                                }
                                                            />
                                                        ) : (
                                                            row[cell.jsonField]
                                                        )}
                                                        
                                                    </TableCell>
                                                );
                                            })}
                                        </TableRow>
                                    );
                                })}
                            </TableBody>
                        </Table>
                    )}
                </TableContainer>
                <Paginator
                    onPageChange={onPageChange}
                    pagination={{ ...pagination, rowsLength: rows.length }}
                />
            </Box>
        </Box>
    );
}
