/* eslint-disable @typescript-eslint/no-explicit-any */
import { Alert, CircularProgress, FormControl, InputLabel, MenuItem, Select, SelectChangeEvent, Snackbar, TextField, Typography, colors } from '@mui/material';
import LoadingButton from '@mui/lab/LoadingButton';
import FormContainer from '../../components/FormContainer';
import { useState } from 'react';
import { useSearchParams } from 'react-router-dom';
import { checkPasswordLength, checkPasswordCase, checkPasswordNumber } from '../../validators/PasswordValidator';
import { isCPFValid } from '../../validators/CPFValidator';
import { AdminUserData, useCreateUser } from '../../api/auth/users/mutations';
import { useLogin } from '../../api/auth/login/mutations';
import { PROCEDIMENTOS_CRIMINAIS } from '../../app/Routes';

function Invite() {

  const [searchParams] = useSearchParams();

  const createUser = useCreateUser();
  const loginNewUser = useLogin();
  const [loading, setLoading] = useState(false);

  const [form, setForm] = useState({
    name: '',
    cpf: '',
    profile: '',
    password: '',
    confirmPassword: '',
    inviteId: searchParams.get("token")
  } as AdminUserData);

  const [formErrors, setFormErrors] = useState({
    name: '',
    cpf: '',
    profile: '',
    password: '',
    confirmPassword: ''
  });

  const [errorSnackbar, setErrorSnackbar] = useState(false);
  const [errorMessage, setErrorMessage] = useState("");

  function handleSubmit(event: any) {
    event.preventDefault();
    setLoading(true);
    const isValid = validateForm();

    if (isValid) {
      createUser.mutate(form, {
        onSuccess: () => {
          loginNewUser.mutate({ cpf: form.cpf.replace(/\D/g, ''), password: form.password }, {
            onSuccess: () => {
              setLoading(false);
              window.location.replace(location.protocol + '//' + location.host + PROCEDIMENTOS_CRIMINAIS)
            },
            onError: (e: any) => {
              setErrorMessage(e.response.data.message ?? e.message);
              setErrorSnackbar(true);
              setLoading(false);
            }
          });
        },
        onError: (e: any) => {
          setErrorMessage(e.response.data.message ?? e.message);
          setErrorSnackbar(true);
          setLoading(false);
        }
      })
    } else {
      setLoading(false)
    }
  }

  const handleNameChange = (event: { target: { value: string; }; }) => {
    setFormErrors({ ...formErrors, name: "" });
    setForm({ ...form, name: event.target.value });
  };

  const handleCpfChange = (event: { target: { value: string; }; }) => {
    setFormErrors({ ...formErrors, cpf: "" });
    const numericValue = event.target.value.replace(/\D/g, '');
    setForm({ ...form, cpf: numericValue });

    if (numericValue.length === 11) {
      const formattedNumber = `${numericValue.slice(0, 3)}.${numericValue.slice(
        3,
        6
      )}.${numericValue.slice(6, 9)}-${numericValue.slice(9, 11)}`;
      setForm({ ...form, cpf: formattedNumber });
    }
  };

  const handleProfileSelect = (event: SelectChangeEvent) => {
    setFormErrors({ ...formErrors, profile: "" })
    setForm({ ...form, profile: event.target.value });
  };

  const handlePasswordChange = (event: { target: { value: string; }; }) => {
    setFormErrors({ ...formErrors, password: "" })
    setForm({ ...form, password: event.target.value });
  };

  const handleConfirmPasswordChange = (event: { target: { value: string; }; }) => {
    setFormErrors({ ...formErrors, confirmPassword: "" })
    setForm({ ...form, confirmPassword: event.target.value });
  };

  const validateForm = () => {
    const errors = { ...formErrors };

    errors.name = validateName(form.name);
    errors.cpf = validateCPF(form.cpf);
    errors.profile = validateProfile(form.profile);
    errors.password = validatePassword(form.password);
    errors.confirmPassword = validateRepeatPassword(form.password, form.confirmPassword);

    setFormErrors(errors);

    if (errors.name === "" && errors.cpf === "" && errors.profile === "" && errors.password === "" && errors.confirmPassword === "") {
      return true;
    } else {
      return false;
    }

  };

  const validateName = (name: string) => {
    if (name.trim().length === 0) {
      return 'Por favor, informe um nome válido';
    }
    return '';
  };

  const validateCPF = (cpf: string) => {
    if (!isCPFValid(cpf.replace(/\D/g, ''))) {
      return 'Por favor, informe um CPF válido';
    }
    return '';
  };

  const validateProfile = (profile: string) => {
    if (profile.length === 0) {
      return 'Por favor, selecione um perfil';
    }
    return '';
  };

  const validatePassword = (password: string) => {
    if (!checkPasswordLength(password)) {
      return 'A senha deve conter no mínimo 8 caracteres.';
    }
    if (!checkPasswordCase(password)) {
      return 'A senha deve conter letras maiúsculas e minúsculas.';
    }
    if (!checkPasswordNumber(password)) {
      return 'A senha deve conter números.';
    }
    if (password.length === 0) {
      return 'Por favor, informe uma senha válida';
    }
    return '';
  };

  const validateRepeatPassword = (password: string, repeatPassword: string) => {
    if (repeatPassword.length === 0) {
      return 'Por favor, confirme sua senha';
    }
    if (password !== repeatPassword) {
      return 'As senhas devem coincidir.';
    }
    return '';
  };

  return (
    <FormContainer>
      <div style={{ display: 'flex', flexDirection: 'column', justifyContent: 'flex-start', alignItems: 'center', height: '100%', marginRight: 40 }}>
        <div style={{ marginTop: 50, marginBottom: 50, alignItems: 'center', display: 'flex', flexDirection: 'column' }}>
          <h1 style={{ color: "#FB8500" }} >Bem-vindo à plataforma Vítimas de Crime</h1>
          <p style={{ color: colors.grey[600] }}>Finalize seu cadastro para acessar a plataforma</p>
        </div>
        <form style={{ display: 'flex', flexDirection: 'column', justifyContent: 'center', width: '100%' }} onSubmit={handleSubmit}>
          <TextField
            type="text"
            label="Nome"
            color='warning'
            onChange={e => handleNameChange(e)}
            error={formErrors.name !== ""}
            helperText={formErrors.name}
            value={form.name}
            fullWidth
            sx={{ mb: 4 }}
          />
          <TextField
            type="text"
            color='warning'
            label="CPF"
            error={formErrors.cpf !== ""}
            helperText={formErrors.cpf}
            onChange={e => handleCpfChange(e)}
            value={form.cpf}
            fullWidth
            sx={{ mb: 4 }}
          />
          <FormControl sx={{ mb: 4 }}>
            <InputLabel error={formErrors.profile !== ""} color='warning' id={"profile-label"}>Perfil</InputLabel>
            <Select
              error={formErrors.profile !== ""}
              labelId={"profile-label"}
              label={"Perfil"}
              color='warning'
              value={form.profile}
              onChange={handleProfileSelect}
              MenuProps={{
                sx: {
                  "&& .Mui-selected::after": {
                    opacity: 0
                  }
                }
              }}
            >
              <MenuItem key={"PC"} value={"Polícia Civil"}>Polícia Civil</MenuItem>
              <MenuItem key={"MP"} value={"Ministério Público"} >Ministério Público</MenuItem>
            </Select>
            <Typography sx={{ ml: '14px' }} fontSize={12} color='error'>{formErrors.profile}</Typography>
          </FormControl>
          <TextField
            type="password"
            color='warning'
            label="Senha"
            onChange={e => handlePasswordChange(e)}
            error={formErrors.password !== ""}
            helperText={formErrors.password}
            value={form.password}
            fullWidth
            sx={{ mb: 4 }}
          />
          <TextField
            type="password"
            color='warning'
            label="Confirme a Senha"
            error={formErrors.confirmPassword !== ""}
            helperText={formErrors.confirmPassword}
            onChange={e => handleConfirmPasswordChange(e)}
            value={form.confirmPassword}
            fullWidth
            sx={{ mb: 4 }}
          />
          <LoadingButton style={{
            padding: '15px',
            borderRadius: '30px',
            background: 'linear-gradient(to right, #FFB703, #FB8500)',
            boxShadow: '0px 2px 4px rgba(0, 0, 0.4, 0.3)',
            color: 'white',
            width: '50%',
            height: '3rem',
            alignSelf: 'center',
            borderColor: '#FFB703',
          }}
            loadingIndicator={<CircularProgress style={{ color: 'white' }} size={26} />}
            type="submit"
            loading={loading}>
            {!loading ? "CADASTRAR" : " "}
          </LoadingButton>
        </form>
        <Snackbar open={errorSnackbar} autoHideDuration={6000} onClose={() => setErrorSnackbar(false)}>
          <Alert onClose={() => setErrorSnackbar(false)} severity="error" sx={{ width: '100%' }}>
            {errorMessage}
          </Alert>
        </Snackbar>
      </div>
    </FormContainer>
  );
}

export default Invite;
