import React from 'react';
import Button from '@mui/material/Button';
import ArrowDownwardIcon from '@mui/icons-material/ArrowDownward';
import './ProcedureDetails.css';
import { useParams } from 'react-router-dom';
import { ProcedureData, useGetCriminalProcedureByID } from '../../api/public/criminal-procedures/queries';
import { useGetUserByID } from '../../api/auth/users/queries';
import { useGetDocumentForCriminalProcedure } from '../../api/public/files/queries';
import { PatchReviewData, usePatchReviewer } from '../../api/public/criminal-procedures/mutations';
import { useAuth } from '../../hooks/useAuth';
import { useDownloadFile } from '../../api/public/files/mutations';

interface Document {
  label: string;
  id: string;
}

interface CaseDetails {
  processoNumber: string;
  nome: string;
  cpf: string;
  telefone: string;
  data: string;
  tipoDeCrime: string;
  cidade: string;
  status: string;
  descricao: string;
  documentos: Document[];
  lidoPor: string;
}

const CaseDetails: React.FC<CaseDetails> = ({
  processoNumber,
  nome,
  cpf,
  telefone,
  data,
  tipoDeCrime,
  cidade,
  status,
  descricao,
  documentos,
  lidoPor
}) => {
  const { id } = useAuth();
  const patchReviewer = usePatchReviewer();
  const downloadFile = useDownloadFile();

  const handleReview = () => {
    patchReviewer.mutate({ procedureNumber: processoNumber, reviewerId: id } as PatchReviewData);
  }

  const handleDownload = (id: string, name: string) => {
    downloadFile.mutate(id, {
      onSuccess: (response) => {
        const href = URL.createObjectURL(response.data);
        const link = document.createElement('a');
        link.href = href;
        link.setAttribute('download', name);
        document.body.appendChild(link);
        link.click();
        document.body.removeChild(link);
        URL.revokeObjectURL(href);
      }
    });
  }

  return (
    <div className="processo-container">
      <div style={{padding: '20px'}}>
        <div className="processo-title">
          <h1 className="processo-number">{tipoDeCrime === "PROCESS" ? "Processo" : "Ocorrência"} nº {processoNumber}</h1>
          <button onClick={() => handleReview()} className="mark-as-read">Marcar como lido</button>
        </div>
        <div className="details-grid">
          <div className="detail-item">
            <span className="detail-title">Nome</span>
            <span className="detail-content">{nome}</span>
          </div>
          <div className="detail-item">
            <span className="detail-title">CPF</span>
            <span className="detail-content">{cpf}</span>
          </div>
          <div className="detail-item">
            <span className="detail-title">Telefone</span>
            <span className="detail-content">{telefone}</span>
          </div>
          <div className="detail-item">
            <span className="detail-title">Data</span>
            <span className="detail-content">{data}</span>
          </div>
        </div>
        <div className="details-grid">
          <div className="detail-item">
            <span className="detail-title">Tipo de Crime</span>
            <span className="detail-content">{tipoDeCrime}</span>
          </div>
          <div className="detail-item">
            <span className="detail-title">Cidade</span>
            <span className="detail-content">{cidade}</span>
          </div>
          <div className="detail-item">
            <span className="detail-title">Status</span>
            <span className="detail-content">{status}</span>
          </div>
          <div className="detail-item">
            <span className="detail-title">Lido por</span>
            <span className="detail-content">{lidoPor}</span>
          </div>
        </div>
        <div className="description-item">
          <span className="description-title">Descrição</span>
          <p className="description-content">{descricao}</p>
        </div>
        <div className="details-grid">
          <span className="document-section">Documentos Anexados</span>
        </div>
        <div style={{ overflowY: 'auto', height: '25rem', width: '98%' }}>
          {documentos.map((doc, index) => (
            <div key={index} className="document-item">
              <span className="document-label">{doc.label}</span>
              <Button onClick={() => handleDownload(doc.id, doc.label)} className="download-icon" variant="text">
                <ArrowDownwardIcon />
              </Button>
            </div>
          ))}
        </div>
      </div>
    </div>
  );
};

const ProcedureDetails: React.FC = () => {
  const { id } = useParams();
  const { data: procedureData = {} as ProcedureData } = useGetCriminalProcedureByID(id!);
  const userData = useGetUserByID(procedureData.victimId).data!;
  const reviewerData = useGetUserByID(procedureData.reviewerId).data!;
  const documentData = useGetDocumentForCriminalProcedure(procedureData.id).data! || [];

  const detalhes: CaseDetails = {
    processoNumber: procedureData.number || ``,
    nome: userData?.name || ``,
    cpf: userData?.cpf || ``,
    telefone: userData?.phoneNumber || ``,
    data: procedureData?.date || ``,
    tipoDeCrime: procedureData?.crime?.name || ``,
    cidade: procedureData?.city?.name || ``,
    status: procedureData?.active ? `Ativo` : `Inativo`,
    descricao: procedureData?.description || ``,
    documentos: documentData.map((document) => { return { label: document.name, id: document.id } }),
    lidoPor: reviewerData?.name || ``,
  };

  return <CaseDetails {...detalhes} />;
};

export default ProcedureDetails;
